@@ -10,12 +10,17 @@
 #include <wchar.h>
 #include <ctype.h>
 
-#ifndef __UCLIBC__
+#ifdef __linux__
+#include <sys/resource.h>
+#endif
+
+#ifndef _CTYPE_H
+#define _CTYPE_H
+#endif
 #ifndef _WCTYPE_H
 #define _WCTYPE_H
 #endif
 #include "../../libc/sysdeps/linux/common/bits/uClibc_ctype.h"
-#endif
 
 /*       0x9 : space  blank */
 /*       0xa : space */
@@ -159,17 +164,17 @@ void output_table(FILE *fp, const char *name, table_data *tbl)
 {
 	size_t i;
 
-	fprintf(fp, "#define WC%s_II_LEN    %7u\n", name, tbl->ii_len);
-	fprintf(fp, "#define WC%s_TI_LEN    %7u\n", name, tbl->ti_len);
-	fprintf(fp, "#define WC%s_UT_LEN    %7u\n", name, tbl->ut_len);
+	fprintf(fp, "#define __LOCALE_DATA_WC%s_II_LEN    %7u\n", name, tbl->ii_len);
+	fprintf(fp, "#define __LOCALE_DATA_WC%s_TI_LEN    %7u\n", name, tbl->ti_len);
+	fprintf(fp, "#define __LOCALE_DATA_WC%s_UT_LEN    %7u\n", name, tbl->ut_len);
 
-	fprintf(fp, "#define WC%s_II_SHIFT  %7u\n", name, tbl->ii_shift);
-	fprintf(fp, "#define WC%s_TI_SHIFT  %7u\n", name, tbl->ti_shift);
+	fprintf(fp, "#define __LOCALE_DATA_WC%s_II_SHIFT  %7u\n", name, tbl->ii_shift);
+	fprintf(fp, "#define __LOCALE_DATA_WC%s_TI_SHIFT  %7u\n", name, tbl->ti_shift);
 
 	fprintf(fp, "\n#ifdef WANT_WC%s_data\n", name);
 
 	i = tbl->ii_len + tbl->ti_len + tbl->ut_len;
-	fprintf(fp, "\nstatic const unsigned char WC%s_data[%zu] = {", name, i);
+	fprintf(fp, "\nstatic const unsigned char __LOCALE_DATA_WC%s_data[%zu] = {", name, i);
 	for (i=0 ; i < tbl->ii_len ; i++) {
 		if (i % 12 == 0) {
 			fprintf(fp, "\n");
@@ -242,6 +247,14 @@ int main(int argc, char **argv)
 	static const char empty_slot[] = "empty_slot";
 	int built = 0;
 
+#ifdef __linux__
+	struct rlimit limit;
+
+	limit.rlim_max = RLIM_INFINITY;
+	limit.rlim_cur = RLIM_INFINITY;
+	setrlimit(RLIMIT_STACK, &limit);
+#endif
+
 #define INIT_TYPENAME(X) typename[__CTYPE_##X] = "C_" #X
 
 	for (i=0 ; i < 16 ; i++) {
@@ -441,8 +454,8 @@ int main(int argc, char **argv)
 			}
 			wct[c/2] |= d;
 
-			l = towlower(c) - c;
-			u = towupper(c) - c;
+			l = (long)(int) towlower(c) - c;
+			u = (long)(int) towupper(c) - c;
 			ult[c] = 0;
 			if (l || u) {
 				if ((l != (short)l) || (u != (short)u)) {
@@ -730,16 +743,16 @@ int main(int argc, char **argv)
 			return EXIT_FAILURE;
 		}
 
-		fprintf(fp, "#define WC_TABLE_DOMAIN_MAX  %#8lx\n\n",
+		fprintf(fp, "#define __LOCALE_DATA_WC_TABLE_DOMAIN_MAX  %#8lx\n\n",
 				(unsigned long) RANGE);
 		output_table(fp, "ctype", &cttable);
 		output_table(fp, "uplow", &ultable);
 	
 
 #warning fix the upper bound on the upper/lower tables... save 200 bytes or so
-		fprintf(fp, "#define WCuplow_diffs  %7u\n", ul_count);
+		fprintf(fp, "#define __LOCALE_DATA_WCuplow_diffs  %7u\n", ul_count);
 		fprintf(fp, "\n#ifdef WANT_WCuplow_diff_data\n\n");
-		fprintf(fp, "\nstatic const short WCuplow_diff_data[%zu] = {",
+		fprintf(fp, "\nstatic const short __LOCALE_DATA_WCuplow_diff_data[%zu] = {",
 			   2 * (size_t) ul_count);
 		for (i=0 ; i < ul_count ; i++) {
 			if (i % 4 == 0) {
